"use strict";
/**
 * Namespace qui regroupe toutes les classes et toutes les fonctions qui permettent l'implémentation des resolvers du schéma GraphQL
 * @namespace GraphQL
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const knex_router_1 = __importDefault(require("../../../db/knex_router"));
class GroupSet extends Set {
    addList(l) {
        for (let elt of l) {
            this.add(elt);
        }
    }
}
exports.GroupSet = GroupSet;
class Tools {
    /**
     * @memberof GraphQL
     * @class Tools
     * @summary Outils intermédiaires LDAP / BDD.
     * @classdesc Cette classe contient des fonctions intermédiaires de recherche dans le LDAP / la BDD.
     */
    constructor() { }
    /**
     * @memberof GraphQL
     * @summary Fonction qui escape l'id donné en argument
     * @arg {string} id - L'identifiant a escape.
     * @return {Promise(Group)} Renvoie l'identifiant escapé.
     * @static
     */
    static escapeID(id) {
        return String(id).replace(' ', '_').replace(/\W/g, '').toLowerCase(); //the REGEXP matches all non-word characters (\W) in a global search (/../g)
    }
    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit l'union de deux ensembles
     * @arg {GroupSet} A - Le premier ensemble.
     * @arg {GroupSet} B - Le deuxieme ensemble.
     * @return {Promise(GroupSet)} Renvoie un nouveau GroupSet contenant l'union de A et B.
     * @static
     */
    static union(A, B) {
        let union = new GroupSet(A);
        for (let b of B) {
            union.add(b);
        }
        return union;
    }
    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit tous les groupes simples dont le user est membre.
     * @arg {userData} data - Données de l'utilisateur.
     * @return {Promise(GroupSet)} Renvoie un GroupSet contenant le nom des groupes simples.
     * @static
     * @async
     */
    static memberOfSimple(data) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
            return new GroupSet(data.members);
        });
    }
    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit tous les groupes simples dont le user est speaker.
     * @arg {userData} data - Données de l'utilisateur.
     * @return {Promise(GroupSet)} Renvoie un GroupSet contenant le nom des groupes simples.
     * @static
     * @async
     */
    static speakerOfSimple(data) {
        return __awaiter(this, void 0, void 0, function* () {
            let speaker = new GroupSet(data.speakers);
            speaker.addList(data.admins);
            return speaker;
        });
    }
    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit tous les groupes simples dont le user est administrateur.
     * @arg {userData} data - Données de l'utilisateur.
     * @return {Promise(GroupSet)} Renvoie un GroupSet contenant le nom des groupes simples.
     * @static
     * @async
     */
    static adminOfSimple(data) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
            return new GroupSet(data.admins);
        });
    }
    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit tous les méta-groupes dont ces groupes sont membres.
     * @arg {GroupSet} groups - Un ensemble de gid des groupes a considérer.
     * @return {Promise(GroupSet)} Renvoie un GroupSet contenant le nom des méta-groupes.
     * @static
     * @async
     */
    static metaGroupsOfGroups(groups) {
        return __awaiter(this, void 0, void 0, function* () {
            let metas = yield knex_router_1.default.select('meta_group_gid').from('metagroup_memberships').whereIn('simple_group_gid', groups);
            return new GroupSet(metas.map(elt => {
                return elt.meta_group_gid;
            }));
        });
    }
    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit tous les groupes (simples ou méta) dont le user est membre.
     * @arg {userData} data - Données de l'utilisateur.
     * @return {Promise(GroupCollection)} Renvoie une GroupCollection contenant le nom des groupes.
     * @static
     * @async
     */
    static memberOf(data) {
        return __awaiter(this, void 0, void 0, function* () {
            let simple = yield Tools.memberOfSimple(data);
            return { simpleGroups: simple, metaGroups: yield Tools.metaGroupsOfGroups(simple) };
        });
    }
    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit tous les groupes (simples ou méta) dont le user est speaker.
     * @arg {userData} data - Données de l'utilisateur.
     * @return {Promise(GroupCollection)} Renvoie une GroupCollection contenant le nom des groupes.
     * @static
     * @async
     */
    static speakerOf(data) {
        return __awaiter(this, void 0, void 0, function* () {
            let speaker = yield Tools.speakerOfSimple(data);
            let admin = yield Tools.adminOfSimple(data);
            return { simpleGroups: speaker, metaGroups: yield Tools.metaGroupsOfGroups(admin) };
        });
    }
    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit tous les groupes (simples ou méta) dont le user est administrateur.
     * @arg {userData} data - Données de l'utilisateur.
     * @return {Promise(GroupCollection)} Renvoie une GroupCollection contenant le nom des groupes.
     * @static
     * @async
     */
    static adminOf(data) {
        return __awaiter(this, void 0, void 0, function* () {
            let simple = yield Tools.adminOfSimple(data);
            return { simpleGroups: simple, metaGroups: yield Tools.metaGroupsOfGroups(simple) };
        });
    }
    /**
     * @memberof GraphQL
     * @summary Fonction qui fait un parcours en profondeur de l'arbre de racine `gid` et renvoie la liste de tous les noeuds.
     * @arg {string} gid - Identifiant du groupe, supposé valide.
     * @return {Promise(string[])} Renvoie une liste contenant le nom des groupes dans cet arbre.
     * @static
     * @async
     */
    static DFS(gid) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = [gid];
            let children = yield knex_router_1.default.select('gid').from('groups').where('parent_gid', gid);
            for (let child of children) {
                let sub_tree = yield Tools.DFS(child.gid);
                res = res.concat(sub_tree);
            }
            return res;
        });
    }
    /**
     * @memberof GraphQL
     * @summary Fonction qui fait un parcours a profondeur 1 de l'arbre de racine `gid` et renvoie la liste des noeuds.
     * @arg {string} gid - Identifiant du groupe, supposé valide.
     * @return {Promise(string[])} Renvoie une liste contenant le nom des groupes a profondeur 1.
     * @static
     * @async
     */
    static oneDownSearch(gid) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = [gid];
            let children = yield knex_router_1.default.select('gid').from('groups').where('parent_gid', gid);
            for (let child of children) {
                res.push(child.gid);
            }
            return res;
        });
    }
    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoie tous les groupes dont l'utilisateur est supervisor.
     * @desc Utilise {@link Tools.oneDownSearch} pour avoir la profondeur 1 des arbres enracinés en chacun des groupes dont il est admin.
     * @arg {userData} data - Données du user.
     * @return {Promise(GroupCollection)} Renvoie une GroupCollection contenant le nom des groupes.
     * @static
     * @async
     */
    static supervisorOf(data) {
        return __awaiter(this, void 0, void 0, function* () {
            let groups = yield Tools.adminOf(data);
            let simple = new GroupSet();
            let meta = new GroupSet();
            for (let g of groups.simpleGroups) {
                simple.addList(yield Tools.oneDownSearch(g));
            }
            for (let g of groups.metaGroups) {
                meta.addList(yield Tools.oneDownSearch(g));
            }
            return { simpleGroups: simple, metaGroups: meta };
        });
    }
    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoie tous les groupes dont l'utilisateur est viewer.
     * @desc Utilise {@link Tools.oneDownSearch} pour avoir la profondeur 1 des arbres enracinés en chacun des groupes dont il est membre.
     * @arg {userData} data - Données du user.
     * @return {Promise(GroupCollection)} Renvoie une GroupCollection contenant le nom des groupes.
     * @static
     * @async
     */
    static viewerOf(data) {
        return __awaiter(this, void 0, void 0, function* () {
            let groups = yield Tools.memberOf(data);
            let simple = new GroupSet();
            let meta = new GroupSet();
            //Ajouter les groupes dont il est membre + les fils a profondeur 1
            for (let g of groups.simpleGroups) {
                simple.addList(yield Tools.oneDownSearch(g));
            }
            for (let g of groups.metaGroups) {
                meta.addList(yield Tools.oneDownSearch(g));
            }
            //Ajouter les groupes simples qui sont dans ses métagroupes
            let s = yield knex_router_1.default.select('simple_group_gid').from('metagroup_memberships').whereIn('meta_group_gid', groups.metaGroups);
            simple.addList(s.map(elt => {
                return elt.simple_group_gid;
            }));
            //TODO : rajouter les VisibilityEdges
            return { simpleGroups: simple, metaGroups: meta };
        });
    }
}
exports.Tools = Tools;
//# sourceMappingURL=tools.js.map