"use strict";
/**
 * @file Resolvers pour tous les types de messages
 * @author akka, ofacklam
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const users_1 = require("./users");
const knex_router_1 = __importDefault(require("../../../db/knex_router"));
const apollo_server_core_1 = require("apollo-server-core");
class Message {
    /**
     * @memberof GraphQL
     * @class Message
     * @summary Resolvers des messages
     * @classdesc Une classe abstraite représentant l'interface Message du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} mid - Identifiant du message, supposé valide.
     * @rights dépend du type de message
     * @abstract
     */
    constructor(mid) {
        this.mid = mid;
        this.m_dataLoaded = false;
    }
    /**
     * Ci-dessous les resolvers a proprement parler.
     */
    /**
     * @memberof GraphQL.Message#
     * @function __resolveType
     * @summary Renvoie quel type de Message c'est
     * @return {string}
     * @rights same as message object
     */
    __resolveType() {
        if (this instanceof Announcement) {
            return "Announcement";
        }
        else if (this instanceof Event) {
            return "Event";
        }
        else if (this instanceof Question) {
            return "Question";
        }
        else if (this instanceof Answer) {
            return "Answer";
        }
        else if (this instanceof PrivatePost) {
            return "PrivatePost";
        }
        else {
            throw new apollo_server_core_1.ApolloError("Bad message type");
        }
    }
    /**
     * @memberof GraphQL.Message#
     * @function createdAt
     * @summary Renvoie la date de création
     * @return {Promise(string)}
     * @rights same as message object
     * @async
     */
    createdAt(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_createdAt;
        });
    }
    /**
     * @memberof GraphQL.Message#
     * @function updatedAt
     * @summary Renvoie la date de mise a jour
     * @return {Promise(string)}
     * @rights same as message object
     * @async
     */
    updatedAt(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_updatedAt;
        });
    }
    /**
     * @memberof GraphQL.Message#
     * @function title
     * @summary Renvoie le titre du message
     * @return {Promise(string)}
     * @rights same as message object
     * @async
     */
    title(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_title;
        });
    }
    /**
     * @memberof GraphQL.Message#
     * @function content
     * @summary Renvoie le contenu du message
     * @return {Promise(string)}
     * @rights same as message object
     * @async
     */
    content(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_content;
        });
    }
}
exports.Message = Message;
class Announcement extends Message {
    /**
     * @memberof GraphQL
     * @class Announcement
     * @extends GraphQL.Message
     * @summary Resolvers des announcements
     * @classdesc Une classe représentant le type Announcement du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} mid - Identifiant du message, supposé valide.
     * @rights membre d'un groupe author ou d'un groupe recipient
     */
    constructor(mid) {
        super(mid);
    }
    /**
     * @memberof GraphQL.Announcement#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer l'annonce correspondante
     * @arg {number} mid - Identifiant du message, sans hypothese sur la validité.
     * @returns {Promise(Announcement)} - Renvoie le Announcement créé, ou null en cas d'erreur.
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     * @static
     */
    static tryCreate(mid) {
        return __awaiter(this, void 0, void 0, function* () {
            let m = new Announcement(mid);
            if (yield m.fetchData()) {
                return m;
            }
            return null;
        });
    }
    /**
     * @memberof GraphQL.Announcement#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur cet announcement dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    fetchData() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.m_dataLoaded) {
                let data = yield knex_router_1.default.select('created_at', 'updated_at', 'title', 'content', 
                //'importance', 
                'views').from('messages_announcements').where('mid', this.mid);
                if (data.length > 0) {
                    let m = data[0];
                    this.m_createdAt = m.created_at;
                    this.m_updatedAt = m.updated_at;
                    this.m_title = m.title;
                    this.m_content = m.content;
                    //this.m_importance = m.importance;
                    this.m_views = m.views;
                    this.m_dataLoaded = true;
                    return true;
                }
                else {
                    return false;
                }
            }
            return true;
        });
    }
    /**
     * Ci-dessous les resolvers a proprement parler.
     */
    /**
     * @memberof GraphQL.Announcement#
     * @function authors
     * @summary Renvoie les groupes auteurs
     * @return {Promise(Group[])}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    authors(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
            /*return knex.select({ uid: 'group' }).from('group_message_relationships')
                .where('message', messageID).whereIn('status', ['host', 'publish']);*/
        });
    }
    /**
     * @memberof GraphQL.Announcement#
     * @function recipients
     * @summary Renvoie les groupes destinataires
     * @return {Promise(Group[])}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    recipients(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
            /*return knex.select({ uid: 'group' }).from('group_message_relationships')
                .where('message', messageID).where('status', 'recieve');*/
        });
    }
    /**
     * @memberof GraphQL.Announcement#
     * @function importance
     * @summary Renvoie l'importance
     * @return {Promise(number)}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    importance(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_importance;
        });
    }
    /**
     * @memberof GraphQL.Announcement#
     * @function views
     * @summary Renvoie le nombre de vues
     * @return {Promise(number)}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    views(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_views;
        });
    }
    /**
     * @memberof GraphQL.Announcement#
     * @function forEvent
     * @summary Renvoie l'evenement correspondant, s'il existe
     * @return {Promise(Event)}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    forEvent(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            let data = yield knex_router_1.default.select('mid').from('messages_events').where('for_announcement', this.mid);
            if (data.length > 0) {
                return new Event(data[0].mid);
            }
            return null;
        });
    }
}
exports.Announcement = Announcement;
class Event extends Message {
    /**
     * @memberof GraphQL
     * @class Event
     * @extends GraphQL.Message
     * @summary Resolvers des events
     * @classdesc Une classe représentant le type Event du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} mid - Identifiant du message, supposé valide.
     * @rights membre d'un groupe author ou d'un groupe recipient
     */
    constructor(mid) {
        super(mid);
    }
    /**
     * @memberof GraphQL.Event#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer l'evenement correspondant
     * @arg {number} mid - Identifiant du message, sans hypothese sur la validité.
     * @returns {Promise(Event)} - Renvoie le Event créé, ou null en cas d'erreur.
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     * @static
     */
    static tryCreate(mid) {
        return __awaiter(this, void 0, void 0, function* () {
            let m = new Event(mid);
            if (yield m.fetchData()) {
                return m;
            }
            return null;
        });
    }
    /**
     * @memberof GraphQL.Event#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur cet event dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    fetchData() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.m_dataLoaded) {
                let data = yield knex_router_1.default.select('created_at', 'updated_at', 'title', 'content', 'location', 'start_time', 'end_time', 'for_announcement').from('messages_events').where('mid', this.mid);
                if (data.length > 0) {
                    let m = data[0];
                    this.m_createdAt = m.created_at;
                    this.m_updatedAt = m.updated_at;
                    this.m_title = m.title;
                    this.m_content = m.content;
                    this.m_location = m.location;
                    this.m_startTime = m.start_time;
                    this.m_endTime = m.end_time;
                    this.m_forAnnouncement = m.for_announcement;
                    this.m_dataLoaded = true;
                    return true;
                }
                else {
                    return false;
                }
            }
            return true;
        });
    }
    /**
     * Ci-dessous les resolvers a proprement parler.
     */
    /**
     * @memberof GraphQL.Event#
     * @function authors
     * @summary Renvoie les groupes auteurs
     * @return {Promise(Group[])}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    authors(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
            /*return knex.select({ uid: 'group' }).from('group_message_relationships')
                .where('message', messageID).whereIn('status', ['host', 'publish']);*/
        });
    }
    /**
     * @memberof GraphQL.Event#
     * @function recipients
     * @summary Renvoie les groupes destinataires
     * @return {Promise(Group[])}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    recipients(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
            /*return knex.select({ uid: 'group' }).from('group_message_relationships')
                .where('message', messageID).where('status', 'recieve');*/
        });
    }
    /**
     * @memberof GraphQL.Event#
     * @function location
     * @summary Renvoie l'endroit
     * @return {Promise(string)}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    location(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_location;
        });
    }
    /**
     * @memberof GraphQL.Event#
     * @function startTime
     * @summary Renvoie l'horaire de début
     * @return {Promise(string)}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    startTime(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_startTime;
        });
    }
    /**
     * @memberof GraphQL.Event#
     * @function endTime
     * @summary Renvoie l'horaire de fin
     * @return {Promise(string)}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    endTime(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_endTime;
        });
    }
    /**
     * @memberof GraphQL.Event#
     * @function participatingGroups
     * @summary Renvoie les groupes qui participent
     * @return {Promise(Group[])}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    participatingGroups(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.Event#
     * @function participatingUsers
     * @summary Renvoie les users qui participent
     * @return {Promise(User[])}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    participatingUsers(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.Event#
     * @function forAnnouncement
     * @summary Renvoie l'announcement correspondant, s'il existe
     * @return {Promise(Announcement)}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    forAnnouncement(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            if (this.m_forAnnouncement == 0) { //what is the default ?????
                return null;
            }
            else {
                return new Announcement(this.m_forAnnouncement);
            }
        });
    }
}
exports.Event = Event;
class PrivatePost extends Message {
    /**
     * @memberof GraphQL
     * @class PrivatePost
     * @extends GraphQL.Message
     * @summary Resolvers des posts privés
     * @classdesc Une classe représentant le type PrivatePost du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} mid - Identifiant du message, supposé valide.
     * @rights membre du groupe recipient
     */
    constructor(mid) {
        super(mid);
    }
    /**
     * @memberof GraphQL.PrivatePost#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer le post privé correspondant
     * @arg {number} mid - Identifiant du message, sans hypothese sur la validité.
     * @returns {Promise(PrivatePost)} - Renvoie le PrivatePost créé, ou null en cas d'erreur.
     * @rights membre du groupe recipient
     * @async
     * @static
     */
    static tryCreate(mid) {
        return __awaiter(this, void 0, void 0, function* () {
            let m = new PrivatePost(mid);
            if (yield m.fetchData()) {
                return m;
            }
            return null;
        });
    }
    /**
     * @memberof GraphQL.PrivatePost#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce post privé dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    fetchData() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.m_dataLoaded) {
                let data = yield knex_router_1.default.select('created_at', 'updated_at', 'title', 'content', 'author', 'recipient').from('messages_private_posts').where('mid', this.mid);
                if (data.length > 0) {
                    let m = data[0];
                    this.m_createdAt = m.created_at;
                    this.m_updatedAt = m.updated_at;
                    this.m_title = m.title;
                    this.m_content = m.content;
                    this.m_author = m.author;
                    this.m_recipient = m.recipient;
                    this.m_dataLoaded = true;
                    return true;
                }
                else {
                    return false;
                }
            }
            return true;
        });
    }
    /**
     * Ci-dessous les resolvers a proprement parler.
     */
    /**
     * @memberof GraphQL.PrivatePost#
     * @function author
     * @summary Renvoie le user auteur
     * @return {Promise(User)}
     * @rights membre du groupe recipient
     * @async
     */
    author(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return new users_1.User(this.m_author);
        });
    }
    /**
     * @memberof GraphQL.PrivatePost#
     * @function recipient
     * @summary Renvoie le groupe destinataire
     * @return {Promise(Group)}
     * @rights membre du groupe recipient
     * @async
     */
    recipient(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            throw "Not implemented";
        });
    }
}
exports.PrivatePost = PrivatePost;
class Question extends Message {
    /**
     * @memberof GraphQL
     * @class Question
     * @extends GraphQL.Message
     * @summary Resolvers des questions
     * @classdesc Une classe représentant le type Question du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} mid - Identifiant du message, supposé valide.
     * @rights viewer du groupe recipient
     */
    constructor(mid) {
        super(mid);
    }
    /**
     * @memberof GraphQL.Question#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer la question correspondante
     * @arg {number} mid - Identifiant du message, sans hypothese sur la validité.
     * @returns {Promise(Question)} - Renvoie la Question créée, ou null en cas d'erreur.
     * @rights viewer du groupe recipient
     * @async
     * @static
     */
    static tryCreate(mid) {
        return __awaiter(this, void 0, void 0, function* () {
            let m = new Question(mid);
            if (yield m.fetchData()) {
                return m;
            }
            return null;
        });
    }
    /**
     * @memberof GraphQL.Question#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce post privé dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    fetchData() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.m_dataLoaded) {
                let data = yield knex_router_1.default.select('created_at', 'updated_at', 'title', 'content', 'author', 'recipient').from('messages_questions').where('mid', this.mid);
                if (data.length > 0) {
                    let m = data[0];
                    this.m_createdAt = m.created_at;
                    this.m_updatedAt = m.updated_at;
                    this.m_title = m.title;
                    this.m_content = m.content;
                    this.m_author = m.author;
                    this.m_recipient = m.recipient;
                    this.m_dataLoaded = true;
                    return true;
                }
                else {
                    return false;
                }
            }
            return true;
        });
    }
    /**
     * Ci-dessous les resolvers a proprement parler.
     */
    /**
     * @memberof GraphQL.Question#
     * @function author
     * @summary Renvoie le user auteur
     * @return {Promise(User)}
     * @rights viewer du groupe recipient
     * @async
     */
    author(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return new users_1.User(this.m_author);
        });
    }
    /**
     * @memberof GraphQL.Question#
     * @function recipient
     * @summary Renvoie le groupe destinataire
     * @return {Promise(Group)}
     * @rights viewer du groupe recipient
     * @async
     */
    recipient(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.Question#
     * @function forAnswer
     * @summary Renvoie la réponse correspondante, si elle existe
     * @return {Promise(Answer)}
     * @rights viewer du groupe recipient
     * @async
     */
    forAnswer(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            let data = yield knex_router_1.default.select('mid').from('messages_answers').where('for_question', this.mid);
            if (data.length > 0) {
                return new Answer(data[0].mid);
            }
            return null;
        });
    }
}
exports.Question = Question;
class Answer extends Message {
    /**
     * @memberof GraphQL
     * @class Answer
     * @extends GraphQL.Message
     * @summary Resolvers des réponses
     * @classdesc Une classe représentant le type Answer du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} mid - Identifiant du message, supposé valide.
     * @rights viewer du groupe author
     */
    constructor(mid) {
        super(mid);
    }
    /**
     * @memberof GraphQL.Answer#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer la réponse correspondante
     * @arg {number} mid - Identifiant du message, sans hypothese sur la validité.
     * @returns {Promise(Answer)} - Renvoie la Answer créée, ou null en cas d'erreur.
     * @rights viewer du groupe author
     * @async
     * @static
     */
    static tryCreate(mid) {
        return __awaiter(this, void 0, void 0, function* () {
            let m = new Answer(mid);
            if (yield m.fetchData()) {
                return m;
            }
            return null;
        });
    }
    /**
     * @memberof GraphQL.Answer#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce post privé dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    fetchData() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.m_dataLoaded) {
                let data = yield knex_router_1.default.select('created_at', 'updated_at', 'title', 'content', 'recipient', //<--- WTF ????
                'for_question').from('messages_answers').where('mid', this.mid);
                if (data.length > 0) {
                    let m = data[0];
                    this.m_createdAt = m.created_at;
                    this.m_updatedAt = m.updated_at;
                    this.m_title = m.title;
                    this.m_content = m.content;
                    this.m_author = m.recipient;
                    this.m_forQuestion = m.for_question;
                    this.m_dataLoaded = true;
                    return true;
                }
                else {
                    return false;
                }
            }
            return true;
        });
    }
    /**
     * Ci-dessous les resolvers a proprement parler.
     */
    /**
     * @memberof GraphQL.Answer#
     * @function author
     * @summary Renvoie le groupe auteur
     * @return {Promise(Group)}
     * @rights viewer du groupe author
     * @async
     */
    author(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.Answer#
     * @function forQuestion
     * @summary Renvoie la question correspondante
     * @return {Promise(Question)}
     * @rights viewer du groupe author
     * @async
     */
    forQuestion(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return new Question(this.m_forQuestion);
        });
    }
}
exports.Answer = Answer;
//# sourceMappingURL=messages.js.map