"use strict";
/**
 * @file Resolvers des utilisateurs
 * @author ofacklam
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const groups_1 = require("./groups");
const user_1 = require("../../ldap/export/user");
const tools_1 = require("../models/tools");
class User {
    /**
     * @memberof GraphQL
     * @class User
     * @summary Resolvers des utilisateurs
     * @classdesc Une classe représentant le type User du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {string} uid - Identifiant du user, supposé valide.
     * @rights connectedOrOnplatal
     */
    constructor(uid) {
        this.uid = uid;
        this.m_dataLoaded = false;
    }
    /**
     * @memberof GraphQL.User#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer le user correspondant
     * @arg {string} uid - Identifiant du user, sans hypothese sur la validité.
     * @returns {Promise(User)} - Renvoie le user créé, ou null en cas d'erreur.
     * @rights connectedOrOnplatal
     * @async
     * @static
     */
    static tryCreate(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            let u = new User(uid);
            if (yield u.fetchData()) {
                return u;
            }
            return null;
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce user dans le LDAP, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    fetchData() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.m_dataLoaded) {
                try {
                    let data = yield user_1.User.peek(this.uid);
                    if (data.uid !== this.uid) {
                        throw "Error in search";
                    }
                    this.m_givenName = data.givenName;
                    this.m_lastName = data.lastName;
                    this.m_nickname = data.nickname;
                    this.m_nationality = data.nationality;
                    this.m_birthdate = data.birthdate;
                    this.m_mail = data.mail;
                    this.m_phone = data.phone;
                    this.m_address = data.address;
                    this.m_memberOf = data.members;
                    this.m_speakerOf = data.speakers;
                    this.m_adminOf = data.admins;
                    this.m_likes = data.followers;
                    this.m_dataLoaded = true;
                    return true;
                }
                catch (_a) {
                    return false;
                }
            }
            return true;
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function givenName
     * @summary Renvoie le prénom
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    givenName(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_givenName;
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function lastName
     * @summary Renvoie le nom de famille
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    lastName(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_lastName;
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function nickname
     * @summary Renvoie le surnom
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    nickname(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_nickname;
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function nationality
     * @summary Renvoie la nationalité
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    nationality(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_nationality;
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function birthdate
     * @summary Renvoie la date de naissance
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    birthdate(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_birthdate;
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function promotion
     * @summary Renvoie la promotion
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    promotion(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function mail
     * @summary Renvoie l'adresse mail
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    mail(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_mail;
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function phone
     * @summary Renvoie le numéro de téléphone
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    phone(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_phone;
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function address
     * @summary Renvoie l'adresse
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    address(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_address;
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function memberOf
     * @summary Renvoie les groupes dont il est membre
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    memberOf(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            let simple = new tools_1.GroupSet(this.m_memberOf);
            let meta = yield tools_1.Tools.metaGroupsOfGroups(simple);
            let all;
            for (let gid of simple) {
                all.push(new groups_1.SimpleGroup(gid));
            }
            for (let gid of meta) {
                all.push(new groups_1.MetaGroup(gid));
            }
            return all;
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function speakerOf
     * @summary Renvoie les groupes dont il est speaker
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    speakerOf(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            let speaker = new tools_1.GroupSet(this.m_speakerOf);
            let admin = new tools_1.GroupSet(this.m_adminOf);
            let meta = yield tools_1.Tools.metaGroupsOfGroups(admin);
            let all;
            for (let gid of speaker) {
                all.push(new groups_1.SimpleGroup(gid));
            }
            for (let gid of meta) {
                all.push(new groups_1.MetaGroup(gid));
            }
            return all;
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function adminOf
     * @summary Renvoie les groupes dont il est admin
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    adminOf(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            let simple = new tools_1.GroupSet(this.m_adminOf);
            let meta = yield tools_1.Tools.metaGroupsOfGroups(simple);
            let all;
            for (let gid of simple) {
                all.push(new groups_1.SimpleGroup(gid));
            }
            for (let gid of meta) {
                all.push(new groups_1.MetaGroup(gid));
            }
            return all;
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function likes
     * @summary Renvoie les groupes dont il est sympathisant
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    likes(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            let simple = new tools_1.GroupSet(this.m_likes);
            let meta = yield tools_1.Tools.metaGroupsOfGroups(simple);
            let all;
            for (let gid of simple) {
                all.push(new groups_1.SimpleGroup(gid));
            }
            for (let gid of meta) {
                all.push(new groups_1.MetaGroup(gid));
            }
            return all;
        });
    }
    /**
     * @memberof GraphQL.User#
     * @function questionsFromUser
     * @summary Renvoie les questions que le user a posées
     * @return {Promise(Question[])}
     * @rights viewer of recipient group
     * @async
     */
    questionsFromUser(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            throw "Not implemented";
        });
    }
}
exports.User = User;
//# sourceMappingURL=users.js.map