"use strict";
/**
 * @file Résolveurs des requêtes GraphQL.
 * @author akka vodol, ofacklam
 * @memberof GraphQL
 * @desc On ne définit dans ce fichier que les resolvers pour les types Query et Mutations.
 * En effet on exploite les default resolvers fournis par Apollo Server (le deuxième point) :
 *
 * > Explicit resolvers are not needed for every type, since Apollo Server provides a default that can perform two actions depending on the contents of parent:
 * > - Return the property from parent with the relevant field name
 * > - Calls a function on parent with the relevant field name and provide the remaining resolver parameters as arguments
 * https://www.apollographql.com/docs/apollo-server/v2/essentials/data.html#default
 *
 * Dans notre cas, le type de retour de chaque resolver est un objet JS (Group, Message, Request ou User)
 * défini dans un des fichiers de `./object_resolvers/` ; donc les méthodes des classes Group, Message, Request et User
 * dont le prototype est (parent, args, context: Context, info) sont utilisées comme resolvers de l'objet GraphQL respectif.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const apollo_server_core_1 = require("apollo-server-core");
const requests_1 = require("./object_resolvers/requests");
const tools_1 = require("./models/tools");
exports.resolvers = {
    Query: {
        // User queries de base
        // @rights connectedOrOnplatal
        user: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isConnectedOrOnplatal()) {
                    return context.models.user.getUser(args.uid);
                }
                throw new apollo_server_core_1.AuthenticationError("Not connected or on-platal");
            });
        },
        // Group queries de base
        // @rights connectedOrOnplatal
        group: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isConnectedOrOnplatal()) {
                    return context.models.group.getGroup(args.gid);
                }
                throw new apollo_server_core_1.AuthenticationError("Not connected or on-platal");
            });
        },
        // @rights connectedOrOnplatal
        simpleGroup: function (obj, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isConnectedOrOnplatal()) {
                    return context.models.group.getSimpleGroup(args.gid);
                }
                throw new apollo_server_core_1.AuthenticationError("Not connected or on-platal");
            });
        },
        // @rights connectedOrOnplatal
        metaGroup: function (obj, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isConnectedOrOnplatal()) {
                    return context.models.group.getMetaGroup(args.gid);
                }
                throw new apollo_server_core_1.AuthenticationError("Not connected or on-platal");
            });
        },
        // Message queries de base
        //message -> not implemented -> message(mid: ID!): Message
        // @rights membre d'un groupe author ou d'un groupe recipient
        announcement: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                //TODO : verifier les autorisations
                throw "Not implemented";
                return context.models.message.getAnnouncement(args.mid);
                throw new apollo_server_core_1.AuthenticationError("Not connected or on-platal");
            });
        },
        // @rights membre d'un groupe author ou d'un groupe recipient
        event: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                //TODO : verifier les autorisations
                throw "Not implemented";
                return context.models.message.getEvent(args.mid);
                throw new apollo_server_core_1.AuthenticationError("Not connected or on-platal");
            });
        },
        // @rights membre du groupe recipient
        privatePost: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                //TODO : verifier les autorisations
                throw "Not implemented";
                return context.models.message.getPrivatePost(args.mid);
                throw new apollo_server_core_1.AuthenticationError("Not connected or on-platal");
            });
        },
        // @rights viewer du groupe recipient
        question: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                //TODO : verifier les autorisations
                throw "Not implemented";
                return context.models.message.getQuestion(args.mid);
                throw new apollo_server_core_1.AuthenticationError("Not connected or on-platal");
            });
        },
        // @rights viewer du groupe author
        answer: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                //TODO : verifier les autorisations
                throw "Not implemented";
                return context.models.message.getAnswer(args.mid);
                throw new apollo_server_core_1.AuthenticationError("Not connected or on-platal");
            });
        },
        // Request queries de base
        //request -> not implemented -> request(rid: ID!): Request
        // @rights admin du groupe destinaire ou le user émetteur
        userJoinGroupRequest: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                //TODO : verifier les autorisations
                throw "Not implemented";
                return context.models.request.getUserJoinGroupRequest(args.rid);
                throw new apollo_server_core_1.AuthenticationError("Not connected or on-platal");
            });
        },
        // @rights admin du groupe émetteur ou destinataire
        groupJoinMetagroupRequest: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                //TODO : verifier les autorisations
                throw "Not implemented";
                return context.models.request.getGroupJoinMetagroupRequest(args.rid);
                throw new apollo_server_core_1.AuthenticationError("Not connected or on-platal");
            });
        },
        // @rights admin du groupe émetteur ou destinataire
        groupCoauthorEventRequest: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                //TODO : verifier les autorisations
                throw "Not implemented";
                return context.models.request.getGroupCoauthorEventRequest(args.rid);
                throw new apollo_server_core_1.AuthenticationError("Not connected or on-platal");
            });
        },
        // Tous les Messages visibles par un utilisateur(dont le uid, et donc les autorisations, est passé par context)
        // @rights member of groups
        allMessages: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                let groups = context.models.auth.groupsMember();
                return context.models.message.getAllMessages(groups);
            });
        },
        // @rights member of groups
        allAnnouncements: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                let groups = context.models.auth.groupsMember();
                return context.models.message.getAllAnnouncements(groups);
            });
        },
        // @rights member of groups
        allEvents: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                let groups = context.models.auth.groupsMember();
                return context.models.message.getAllEvents(groups);
            });
        },
        // @rights member of groups
        allPrivatePosts: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                let groups = context.models.auth.groupsMember();
                return context.models.message.getAllPrivatePosts(groups);
            });
        },
        // Tous les Groupes visibles par un utilisateur.
        // Correspondrait au sous - champ "viewerOf" de User, volontairement non - défini comme tel.Tous les autres cas de figure sont couverts par les sous - champs "<permission>Of" de User
        // @rights viewer of groups
        allGroups: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                let visibleGroupCollection = context.models.auth.groupsViewer();
                return context.models.group.getAllGroupsByCollection(visibleGroupCollection);
            });
        },
        // @rights viewer of groups
        allSimpleGroups: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                let visibleGroupCollection = context.models.auth.groupsViewer();
                return context.models.group.getAllSimpleGroups(visibleGroupCollection.simpleGroups);
            });
        },
        // @rights supervisor of groups
        supervisorOf: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                let supervisorGroupCollection = context.models.auth.groupsSupervisor();
                return context.models.group.getAllGroupsByCollection(supervisorGroupCollection);
            });
        },
        // TOL
        // @rights connectedOrOnplatal
        searchTOL: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isConnectedOrOnplatal()) {
                    return context.models.user.searchTOL(args);
                }
                throw new apollo_server_core_1.AuthenticationError("Not connected or on-platal");
            });
        }
    },
    Mutation: {
        // Groups - independent mutations
        // @rights authenticated
        editProfile: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isAuthenticated()) {
                    return context.models.user.editProfile(args);
                }
                throw new apollo_server_core_1.AuthenticationError("Not authenticated");
            });
        },
        // Viewer mutations
        // @rights viewer
        likeGroup: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isViewer(args.gid)) {
                    return context.models.group.likeGroup(args.gid);
                }
                throw new apollo_server_core_1.AuthenticationError("Not a viewer");
            });
        },
        // @rights viewer
        unlikeGroup: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isViewer(args.gid)) {
                    return context.models.group.unlikeGroup(args.gid);
                }
                throw new apollo_server_core_1.AuthenticationError("Not a viewer");
            });
        },
        // @rights member d'un groupe author ou recipient
        userParticipate: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                //TODO : Vérifier les autorisations
                return context.models.message.userParticipate(context.user.uid, args.forEvent);
                throw new apollo_server_core_1.AuthenticationError("Not a viewer");
            });
        },
        // @rights member d'un groupe author ou recipient
        userUnparticipate: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                //TODO : Vérifier les autorisations
                return context.models.message.userUnparticipate(context.user.uid, args.forEvent);
                throw new apollo_server_core_1.AuthenticationError("Not a viewer");
            });
        },
        // @rights viewer
        userRequestJoinGroup: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isViewer(args.toGroup)) {
                    return context.models.request.userRequestJoinGroup(args.toGroup, args.comment);
                }
                throw new apollo_server_core_1.AuthenticationError("Not a viewer");
            });
        },
        // @rights viewer
        createQuestion: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isViewer(args.toGroup)) {
                    return context.models.message.createQuestion(args.toGroup, args.title, args.content);
                }
                throw new apollo_server_core_1.AuthenticationError("Not a viewer");
            });
        },
        // @rights viewer du groupe et author de la question
        editQuestion: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                // TODO : Vérifier qu'il est l'auteur de la question et viewer
                return context.models.message.editQuestion(args.questionToEdit, args.title, args.content);
                throw new apollo_server_core_1.AuthenticationError("Not a viewer");
            });
        },
        // @rights viewer du groupe et author de la question
        removeQuestion: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                // TODO : Vérifier qu'il est l'auteur de la question et viewer
                return context.models.message.removeQuestion(args.questionToRemove);
                throw new apollo_server_core_1.AuthenticationError("Not a viewer");
            });
        },
        // Member mutations
        // @rights member
        userLeaveGroup: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isMember(args.gid)) {
                    return context.models.group.userLeaveGroup(args.gid);
                }
                throw new apollo_server_core_1.AuthenticationError("Not a member");
            });
        },
        // @rights member
        createPrivatePost: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isMember(args.toGroup)) {
                    return context.models.message.createPrivatePost(args.toGroup, args.title, args.content);
                }
                throw new apollo_server_core_1.AuthenticationError("Not a member");
            });
        },
        // @rights member du groupe et author du post
        editPrivatePost: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                // TODO : Vérifier qu'il est l'auteur du post et member
                return context.models.message.editPrivatePost(args.privatePostToEdit, args.title, args.content);
                throw new apollo_server_core_1.AuthenticationError("Not a member");
            });
        },
        // @rights member du groupe et author du post
        removePrivatePost: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                // TODO : Vérifier qu'il est l'auteur du post et member
                return context.models.message.removePrivatePost(args.privatePostToRemove);
                throw new apollo_server_core_1.AuthenticationError("Not a member");
            });
        },
        // Speaker mutations
        // @rights speaker
        writePostsSummary: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isSpeaker(args.forGroup)) {
                    return context.models.group.writePostsSummary(args.forGroup, args.content);
                }
                throw new apollo_server_core_1.AuthenticationError("Not a speaker");
            });
        },
        // @rights speaker
        groupParticipate: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isSpeaker(args.gid)) {
                    return context.models.message.groupParticipate(args.gid, args.forEvent);
                }
                throw new apollo_server_core_1.AuthenticationError("Not a speaker");
            });
        },
        // @rights speaker
        groupUnparticipate: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isSpeaker(args.gid)) {
                    return context.models.message.groupUnparticipate(args.gid, args.forEvent);
                }
                throw new apollo_server_core_1.AuthenticationError("Not a speaker");
            });
        },
        // @rights speaker du groupe émetteur
        groupRequestCoauthorEvent: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isSpeaker(args.fromGroup)) {
                    return context.models.request.groupRequestCoauthorEvent(args.fromGroup, args.toGroup, args.forEvent, args.comment);
                }
                throw new apollo_server_core_1.AuthenticationError("Not a speaker");
            });
        },
        // @rights speaker du groupe émetteur
        createAnnouncement: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isSpeaker(args.fromGroup)) {
                    return context.models.message.createAnnouncement(args.fromGroup, new tools_1.GroupSet(args.toGroups), args.title, args.content, args.forEvent);
                }
                throw new apollo_server_core_1.AuthenticationError("Not a speaker");
            });
        },
        // @rights speaker du groupe émetteur
        editAnnouncement: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                // TODO : Vérifier les autorisations.
                return context.models.message.editAnnouncement(args.announcementToEdit, args.title, args.content, args.forEvent);
                throw new apollo_server_core_1.AuthenticationError("Not a speaker");
            });
        },
        // @rights speaker du groupe émetteur
        removeAnnouncement: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                // TODO : Vérifier les autorisations
                return context.models.message.removeAnnouncement(args.announcementToRemove);
                throw new apollo_server_core_1.AuthenticationError("Not a speaker");
            });
        },
        // @rights speaker du groupe émetteur
        createEvent: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isSpeaker(args.fromGroup)) {
                    return context.models.message.createEvent(args.fromGroup, new tools_1.GroupSet(args.toGroups), args.title, args.content, args.location, args.startTime, args.endTime, args.forAnnouncement);
                }
                throw new apollo_server_core_1.AuthenticationError("Not a speaker");
            });
        },
        // @rights speaker du groupe émetteur
        editEvent: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                // TODO : Vérifier les autorisations.
                return context.models.message.editEvent(args.eventToEdit, args.title, args.content, args.location, args.startTime, args.endTime, args.forAnnouncement);
                throw new apollo_server_core_1.AuthenticationError("Not a speaker");
            });
        },
        // @rights speaker du groupe émetteur
        removeEvent: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                // TODO : Vérifier les autorisations
                return context.models.message.removeEvent(args.eventToRemove);
                throw new apollo_server_core_1.AuthenticationError("Not a speaker");
            });
        },
        // @rights speaker du groupe
        createAnswer: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                // TODO : Vérifier les autorisations.
                return context.models.message.createAnswer(args.forQuestion, args.title, args.content);
                throw new apollo_server_core_1.AuthenticationError("Not a speaker");
            });
        },
        // @rights speaker du groupe
        editAnswer: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                // TODO : Vérifier les autorisations.
                return context.models.message.editAnswer(args.answerToEdit, args.title, args.content);
                throw new apollo_server_core_1.AuthenticationError("Not a speaker");
            });
        },
        // @rights speaker du groupe
        removeAnswer: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                // TODO : Vérifier les autorisations
                return context.models.message.removeAnswer(args.answerToRemove);
                throw new apollo_server_core_1.AuthenticationError("Not a speaker");
            });
        },
        // Admin mutations
        // @rights admin of parent group
        createSubgroup: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isAdmin(args.fromGroup)) {
                    return context.models.group.createSubgroup(args);
                }
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin
        makeAdmin: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isAdmin(args.forGroup)) {
                    return context.models.group.makeAdmin(args.forGroup, args.uid);
                }
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin
        unmakeAdmin: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isAdmin(args.forGroup)) {
                    return context.models.group.unmakeAdmin(args.forGroup, args.uid);
                }
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin
        makeSpeaker: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isAdmin(args.forGroup)) {
                    return context.models.group.makeSpeaker(args.forGroup, args.uid);
                }
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin
        unmakeSpeaker: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isAdmin(args.forGroup)) {
                    return context.models.group.unmakeSpeaker(args.forGroup, args.uid);
                }
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin
        editGroup: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isAdmin(args.forGroup)) {
                    return context.models.group.editGroup(args);
                }
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin
        addVisibilityEdge: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isAdmin(args.forGroup)) {
                    return context.models.group.addVisibilityEdge(args.forGroup, args.visibleBy);
                }
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin
        removeVisibilityEdge: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isAdmin(args.forGroup)) {
                    return context.models.group.removeVisibilityEdge(args.forGroup, args.visibleBy);
                }
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin du groupe émetteur
        groupRequestJoinMetagroup: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isAdmin(args.fromGroup)) {
                    return context.models.request.groupRequestJoinMetagroup(args.fromGroup, args.toMetagroup, args.comment);
                }
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin du groupe destinataire
        acceptUserJoinRequest: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                let req = yield requests_1.UserJoinGroup.tryCreate(args.request);
                throw "Not implemented";
                //TODO : Vérifier les autorisations
                //if (context.models.auth.isAdmin(req.to)) {
                return context.models.request.acceptUserJoinRequest(req, args.comment);
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin du groupe destinataire
        acceptGroupJoinRequest: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                let req = yield requests_1.GroupJoinMetagroup.tryCreate(args.request);
                throw "Not implemented";
                //TODO : Vérifier les autorisations
                //if (context.models.auth.isAdmin(req.to)) {
                return context.models.request.acceptGroupJoinRequest(req, args.comment);
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin du groupe destinataire
        refuseUserJoinRequest: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                let req = yield requests_1.UserJoinGroup.tryCreate(args.request);
                throw "Not implemented";
                //TODO : Vérifier les autorisations
                //if (context.models.auth.isAdmin(req.to)) {
                return context.models.request.refuseUserJoinRequest(req, args.comment);
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin du groupe destinataire
        refuseGroupJoinRequest: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                let req = yield requests_1.GroupJoinMetagroup.tryCreate(args.request);
                throw "Not implemented";
                //TODO : Vérifier les autorisations
                //if (context.models.auth.isAdmin(req.to)) {
                return context.models.request.refuseGroupJoinRequest(req, args.comment);
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin
        removeUser: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isAdmin(args.fromGroup)) {
                    return context.models.group.removeUser(args.fromGroup, args.uid);
                }
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin du groupe destinataire
        acceptGroupCoauthorEventRequest: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                // Pour l'instant, ce n'est pas a implémenter...
                let req = yield requests_1.GroupCoauthorEvent.tryCreate(args.request);
                throw "Not implemented";
                //TODO : Vérifier les autorisations
                //if (context.models.auth.isAdmin(req.to)) {
                return context.models.request.acceptGroupCoauthorEventRequest(req, args.comment);
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin du groupe destinataire
        refuseGroupCoauthorEventRequest: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                // Pour l'instant, ce n'est pas a implémenter...
                let req = yield requests_1.GroupCoauthorEvent.tryCreate(args.request);
                throw "Not implemented";
                //TODO : Vérifier les autorisations
                //if (context.models.auth.isAdmin(req.to)) {
                return context.models.request.refuseGroupCoauthorEventRequest(req, args.comment);
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin du groupe
        censorQuestion: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                //TODO : Vérifier les autorisations
                return context.models.message.censorQuestion(args.questionToCensor);
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin du groupe
        censorAnswer: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                //TODO : Vérifier les autorisations
                return context.models.message.censorAnswer(args.answerToCensor);
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin du groupe
        censorPrivatePost: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                //TODO : Vérifier les autorisations
                return context.models.message.censorPrivatePost(args.privatePostToCensor);
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin d'un groupe author
        censorAnnouncement: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                //TODO : Vérifier les autorisations
                return context.models.message.censorAnnouncement(args.announcementToCensor);
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // @rights admin d'un groupe author
        censorEvent: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                throw "Not implemented";
                //TODO : Vérifier les autorisations
                return context.models.message.censorEvent(args.eventToCensor);
                throw new apollo_server_core_1.AuthenticationError("Not an admin");
            });
        },
        // Supervisor mutations
        // @rights supervisor
        takeAdminRights: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isSupervisor(args.forGroup)) {
                    return context.models.group.takeAdminRights(args.forGroup, context.user.uid);
                }
                throw new apollo_server_core_1.AuthenticationError("Not a supervisor");
            });
        },
        // @rights supervisor
        releaseAdminRights: function (root, args, context) {
            return __awaiter(this, void 0, void 0, function* () {
                if (context.models.auth.isSupervisor(args.forGroup)) {
                    return context.models.group.releaseAdminRights(args.forGroup, context.user.uid);
                }
                throw new apollo_server_core_1.AuthenticationError("Not a supervisor");
            });
        }
    }
};
//# sourceMappingURL=resolvers.js.map