"use strict";
/**
 * @file Ce fichier contient la classe de l'API du LDAP qui gère les opérations sur les groupes. Elle est destinée à être exportée pour être utilisée par les resolvers.
 * @author hawkspar
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("../internal/config");
const basics_1 = require("../internal/basics");
const tools_1 = require("../internal/tools");
//------------------------------------------------------------------------------------------------------------------------
// Classes à exporter TBT
//------------------------------------------------------------------------------------------------------------------------
class Group {
    /**
     * @memberof LDAP
     * @class Group
     * @classdesc Cette classe est une des deux classes exportables permettant de faire des opérations sur les groupes.
     * @summary Constructeur vide.
     */
    constructor() { }
    /**
     * @memberof LDAP
     * @summary Fonction qui renvoit toutes les infos relatives à un groupe particulier.
     * @desc Cette fonction utilise {@link Tools.peek} avec l'interface {@link groupData}. Elle rajoute les individus
     * @arg {string} gid - Identifiant du groupe
     * @return {Promise(groupData)} Informations recueillies ; renvoie une liste de dictionnaire avec le profil complet du groupe au format {@link groupData}.
     * @static
     * @async
     */
    static peek(gid) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return tools_1.Tools.peek("group", gid, config_1.groupData);
            }
            catch (err) {
                throw "Erreur lors d'une recherche d'informations sur un groupe.";
            }
        });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui retrouve le groupe qui ressemblent à l'input et qui correspond au type fourni. Etape 0 vers un vrai TOL (Trombino On Line) pour les groupes.
     * @desc Cette fonction utilise {@link Tools.search}.
     * @arg {groupData} input - String entré par l'utilisateur qui ressemble au nom du groupe.
     * @return {Promise(string[])} Liste des gid dont le nom ressemble à l'input.
     * @static
     * @async
    */
    static search(data) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return tools_1.Tools.search("group", data);
            }
            catch (err) {
                throw "Erreur lors de la recherche approximative d'un groupe.";
            }
        });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de rajouter un administrateur à un groupe.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.add}. Le nouvel administrateur ne devient pas membre ou porte-parole du groupe pour autant !
     * @arg {string} uid - Identifiant du membre futur admin
     * @arg {string} gid - Identifiant du groupe
     * @return {boolean} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    static addAdmin(uid, gid) {
        return __awaiter(this, void 0, void 0, function* () { return yield tools_1.Tools.add(uid, gid, "admins"); });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de supprimer un administrateur.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.remove}.
     * Elle ne remonte pas les échelons, car cela permettrait à un admin d'un petit groupe de supprimer un admin d'un grand.
     * @arg {string} uid - Identifiant de l'admin à dégrader, supposé membre
     * @arg {string} gid - Identifiant du groupe
     * @return {boolean} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    static remAdmin(uid, gid) {
        return __awaiter(this, void 0, void 0, function* () { return tools_1.Tools.remove(uid, gid, "admins"); });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de rajouter un porte-parole à un groupe.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.add}. Elle ne rajoute pas l'utilisateur au groupe.
     * @arg {string} uid - Identifiant du membre futur porte-parole
     * @arg {string} gid - Identifiant du groupe
     * @return {boolean} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    static addSpeaker(uid, gid) {
        return __awaiter(this, void 0, void 0, function* () { return yield tools_1.Tools.add(uid, gid, "speakers"); });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de rétrograder un membre du stade de porte-parole d'un groupe au stade d'utilisateur.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.remove}. Elle dégrade aussi d'un éventuel statut d'administrateur.
     * @arg {string} uid - Identifiant de l'admin à dégrader (pas supposé valide)
     * @arg {string} gid - Identifiant du groupe
     * @return {boolean} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    static remSpeaker(uid, gid) {
        return __awaiter(this, void 0, void 0, function* () { return yield tools_1.Tools.remove(uid, gid, "speakers"); });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui permet d'ajouter un utilisateur à un groupe.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.add}.
     * @arg {string} uid - Identifiant de l'utilisateur à ajouter
     * @arg {string} gid - Identifiant du groupe
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    static addMember(uid, gid) {
        return __awaiter(this, void 0, void 0, function* () { return tools_1.Tools.add(uid, gid, "members"); });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de supprimer un membre existant d'un groupe.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.add}.
     * Cette fonction supprime tous les droits de l'utilisateur sur le groupe, mais aussi sur les groupes sources si son statut de membre était hérité.
     * @arg {string} uid - Identifiant de l'ex-membre (pas supposé valide)
     * @arg {string} gid - Identifiant du groupe
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    static remMember(uid, gid) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield tools_1.Tools.remove(uid, gid, "members");
            // Ce qui suit est une suppression récursive du membre dans tous les groupes fils
            let stack = [];
            let res = true;
            let visited = {};
            stack.push(gid);
            while (stack.length > 0) {
                let cur_id = stack.pop();
                if (visited[cur_id] == undefined) {
                    visited[cur_id] = true;
                    res = res && (yield tools_1.Tools.remove(uid, cur_id, "admins")) &&
                        (yield tools_1.Tools.remove(uid, cur_id, "speakers")) &&
                        (yield tools_1.Tools.remove(uid, cur_id, "members"));
                    stack.concat(yield basics_1.Basics.searchSingle("group", config_1.ldapConfig.group.childs, cur_id));
                }
            }
            return res;
        });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui permet d'ajouter un sympathisant à un groupe.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.add}.
     * @arg {string} uid - Identifiant de l'utilisateur à ajouter
     * @arg {string} gid - Identifiant du groupe
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    static addFollower(uid, gid) {
        return __awaiter(this, void 0, void 0, function* () { return tools_1.Tools.add(uid, gid, "followers"); });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de supprimer un sympathisant d'un groupe.
     * @desc Cette fonction fait essentiellement appel à {@link Tools.add}.
     * @arg {string} uid - Identifiant de l'ex-sympathisant (pas supposé valide)
     * @arg {string} gid - Identifiant du groupe
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    static remFollower(uid, gid) {
        return __awaiter(this, void 0, void 0, function* () { return tools_1.Tools.remove(uid, gid, "followers"); });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui créé un nouveau groupe dans le LDAP.
     * @desc Cette fonction fait une utilisation massive d'eval pour anonymiser son code ; c'est mal et cela suppose que beaucoup de soins ont été pris lors de
     * l'escape de ses paramètres. Appelle {@link LDAP.add} et {@link LDAP.change}, mais aussi {@link Tools.add}
     * pour gérer les groupes du nouvel utilisateur. Cettte application permet de rajouter des utilisateurs à toutes les catégories du groupe.
     * @arg {groupData} data - Dictionnaire des informations utilisateurs (voir détail des champs dans ldapConfig.json)
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    static create(data) {
        return __awaiter(this, void 0, void 0, function* () {
            // Calcul d'un dictionnaire d'ajout
            let vals = {};
            // gid de base généré à partir du nom standardisé, pas à partir de l'entrée 'gid' !
            try {
                tools_1.Tools.generateReadableId(data['name']).then(id => {
                    vals[config_1.ldapConfig.key_id] = id;
                    vals[config_1.ldapConfig.group['name']] = id;
                });
            }
            catch (err) {
                throw "Erreur lors de la génération d'un hruid pour créer un nouveau groupe.";
            }
            let gid = vals[config_1.ldapConfig.key_id];
            // Ecriture de toutes les valeurs directement inscrites dans le LDAP
            for (let key_att in data) {
                vals[config_1.ldapConfig.group[key_att]] = data[key_att];
            }
            ;
            // Appel à la fonction de base
            if (!(yield basics_1.Basics.add("group", vals))) {
                throw "Erreur lors de la création d'une nouvelle feuille dans l'arbre des groupes.";
            }
            // Certains champs nécessitent de petits calculs
            let vals2 = {};
            // ?!
            vals2[config_1.ldapConfig.group['password']] = "{CRYPT}" + data['password'];
            // Génération id aléatoire et test contre le LDAP
            try {
                tools_1.Tools.generateId(config_1.ldapConfig.group["idNumber"], "group").then(id => { vals2[config_1.ldapConfig.group['idNumber']] = id; });
            }
            catch (err) {
                throw "Erreur lors de la génération d'un id numérique pour créer un nouveau groupe.";
            }
            // Code root
            vals2[config_1.ldapConfig.group['cleanFullName']] = data['name'].replace(':', ';').toLowerCase().normalize('UFD');
            // Inscription des valeurs calculées par effet de bord
            if (!(yield basics_1.Basics.change("group", gid, "add", vals2))) {
                throw "Erreur lors de l'ajout des valeurs intelligentes du nouveau groupe.";
            }
            ["posixGroup", "brGroup"].forEach(cst => {
                let vals3 = {};
                vals3[config_1.ldapConfig.group['classes']] = cst;
                basics_1.Basics.change("group", gid, "add", vals3).then(res => {
                    if (!res) {
                        throw "Erreur lors de l'ajout des valeurs constantes du nouveau groupe.";
                    }
                });
            });
            // Utilisation des fonctions adaptées pour assurer la cohérence de l'ensemble
            for (let cat in config_1.categories) {
                data[cat].forEach(uid => {
                    tools_1.Tools.add(uid, gid, cat).then(res => {
                        if (!res) {
                            throw "Erreur de l'ajout d'un membre au nouveau groupe.";
                        }
                    });
                });
            }
            return true;
        });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui supprime un groupe du LDAP.
     * @desc Cette fonction commence par gérer les groupes du membre puis le supprime entièrement. A modifier une fois que le LDAP incluerait les groupes administres par une utilisateur.
     * Appelle {@link LDAP.clear} bien sûr, mais aussi {@link Tools.remove} pour gérer les groupes de l'utilisateur sortant.
     * @arg {string} gid - gid de la victime
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    static delete(gid) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                // Gestion des catégories en bloc d'abord
                let profil = yield Group.peek(gid);
                for (let cat of config_1.categories) {
                    profil[config_1.ldapConfig.group[cat]].forEach(function quickPartRemUser(uid) {
                        return __awaiter(this, void 0, void 0, function* () {
                            // Modification des profils de tous les utilisateurs
                            let lg = yield tools_1.Tools.get(uid, "user", cat);
                            // Vérifie que l'utilisateur est pas déjà viré pour users
                            if (lg.includes(gid)) {
                                // Supprime tous les groupes
                                if (!(yield basics_1.Basics.change("user", uid, "del", config_1.ldapConfig.user[cat]))) {
                                    throw "Erreur lors de la suppression de tous les groupes du membre.";
                                }
                                // Les rajoute un par un, sauf pour le groupe supprimé
                                lg.forEach(id => {
                                    if (id != gid) {
                                        tools_1.Tools.add(uid, id, cat).then(res => {
                                            if (!res) {
                                                throw "Erreur lors du ré-ajout des autres groupes";
                                            }
                                        });
                                    }
                                });
                            }
                        });
                    });
                }
                // Elimination
                if (!(yield basics_1.Basics.clear("group", gid))) {
                    throw "Erreur lors de la suppression de la feuille dans l'arbre des groupes.";
                }
                return true;
            }
            catch (err) {
                throw "Erreur lors de l'obtention du profil d'un groupe pour le supprimer.";
            }
        });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui édite un groupe existant dans le LDAP. Sans influence sur ses membres ou admins.
     * @desc Appelle {@link Tools.edit}.
     * @arg {groupData} data - Dictionnaire des informations du groupe.
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    static edit(data) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return tools_1.Tools.edit("group", data);
            }
            catch (err) {
                throw "Erreur lors de la modification d'un groupe.";
            }
        });
    }
}
exports.Group = Group;
//# sourceMappingURL=group.js.map