"use strict";
/**
 * @file Ce fichier contient la classe de l'API du LDAP qui gère les opérations sur les groupes. Elle est destinée à être exportée pour être utilisée par les resolvers.
 * @author hawkspar
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("../internal/config");
exports.userData = config_1.userData;
const basics_1 = require("../internal/basics");
const tools_1 = require("../internal/tools");
class User {
    /**
     * @memberof LDAP
     * @class User
     * @classdesc Cette classe est une des deux classes exportables permettant de faire des opérations sur les utilisateurs.
     * @summary Constructeur vide.
     */
    constructor() { }
    /**
     * @memberof LDAP
     * @summary Fonction qui renvoit les infos de base relatives à un utilisateur particulier.
     * @desc Cette fonction utilise {@link Tools.peek} avec l'interface {@link userData}.
     * @arg {string} uid - Identifiant de l'utilisateur, supposé valide.
     * @return {Promise(userData)} Informations recueillies au format {@link userData}.
     * @static
     * @async
     */
    static peek(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let data = yield tools_1.Tools.peek("user", uid, config_1.userData);
                for (let cat in config_1.categories) {
                    data[cat] = yield basics_1.Basics.searchSingle("group", config_1.ldapConfig.group.key_id, "*", config_1.ldapConfig.group[cat] + "=" + uid);
                }
                return data;
            }
            catch (err) {
                throw "Error while peeking a user.";
            }
        });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui retrouve les uid des paxs validant les critères de recherche. Utiliser {@link User.peek} au cas par cas après pour obtenir les vraies infos.
     * @desc Cette fonction utilise {@link Tools.search}.
     * @arg {userData} data - Dictionnaire contenant les données nécessaires à la recherche. Les valeurs sont celles entrées par l'utilisateur et sont par hypothèse
     * comme des sous-parties compactes des valeurs renvoyées. Tous les champs ci-dessous peuvent être indifféremment des listes (par exemple pour chercher un membre
     * de plusieurs groupes) ou des éléments isolés. Si un champ n'est pas pertinent, le mettre à '' ou undefined.
     * @return {Promise(string[])} gids des profils qui "match" les critères proposés.
     * @static
     * @async
     */
    static search(data) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return tools_1.Tools.search("user", data);
            }
            catch (err) {
                throw "Erreur lors de la recherche approximative d'un utilisateur.";
            }
        });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui créé un nouvel utilisateur dans le LDAP.
     * @desc Appelle {@link LDAP.add} bien sûr, mais aussi {@link Tools.add} pour gérer les groupes du nouvel utilisateur.
     * @arg {userData} data - Dictionnaire des informations utilisateurs. Des erreurs peuvent apparaître si tous les champs ne sont pas remplis.
     * Cette application permet de rejoindre des groupes en masse pour toute catégorie.
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    static create(data) {
        return __awaiter(this, void 0, void 0, function* () {
            // Calcul d'un dictionnaire d'ajout
            let vals = {};
            // uid de base généré à partir de nom et prénom, plus potentiellement promo et un offset
            // MEF mélange de Promise et de fonction standard
            try {
                tools_1.Tools.generateUid(data['givenName'], data['lastName'], data['birthdate']).then(id => { vals[config_1.ldapConfig.key_id] = id; });
            }
            catch (err) {
                throw "Erreur lors de la génération d'un hruid pour un nouvel utilisateur.";
            }
            let uid = vals[config_1.ldapConfig.key_id];
            // Génère une erreur si un champ n'est pas rempli
            for (let key_att in data) {
                // Ecriture de toutes les valeurs uniques
                if (!Array.isArray(data[key_att])) {
                    vals[config_1.ldapConfig.user[key_att]] = data[key_att];
                }
            }
            // Appel à la fonction de base
            if (!(yield basics_1.Basics.add("user", vals))) {
                throw "Erreur de l'ajout de la feuille à l'arbre utilisateur.";
            }
            for (let key_att in data) {
                // Modifications multiples pour avoir plusieurs champs de même type ; boucle sur les attributs multiples
                if (Array.isArray(data[key_att])) {
                    // On rajoute chaque valeur en entrée
                    data[key_att].forEach(val => {
                        let vals2 = {};
                        vals2[config_1.ldapConfig.user[key_att]] = val;
                        basics_1.Basics.change("user", uid, "add", vals2).then(res => {
                            if (!res) {
                                throw "Erreur lors de l'ajout d'une valeur pour un champ à valeurs multiples à la feuille du nouvel utilisateur.";
                            }
                        });
                    });
                }
            }
            // Certains champs nécessitent de petits calculs
            let vals3 = {};
            // ldapConfiguration du mot de passe utilisateur
            // Le préfixe {CRYPT} signifie que le mdp est hashé dans OpenLDAP voir : https://www.openldap.org/doc/admin24/security.html 
            vals3[config_1.ldapConfig.user['password']] = "{CRYPT}" + data['password'];
            // Ecriture d'un surnom s'il y a lieu
            if ((data['nickname'] != undefined) && (data['nickname'] != '')) {
                vals3[config_1.ldapConfig.user['nickname']] = data['nickname'];
            }
            try {
                // Génération id aléatoire unique
                vals3[config_1.ldapConfig.user['id']] = yield tools_1.Tools.generateId(config_1.ldapConfig.user['id'], "user");
            }
            catch (err) {
                throw "Erreur lors de la génération d'un id numérique pour un nouvel utilisateur.";
            }
            // Code root
            vals3[config_1.ldapConfig.user['cleanFullName']] = data['fullName'].replace(':', ';').toLowerCase().normalize('UFD');
            // Inscription des valeurs calculées
            if (!(yield basics_1.Basics.change("user", uid, "add", vals3))) {
                throw "Erreur lors de l'ajout des valeurs calculées à la feuille du nouvel utilisateur.";
            }
            ["posixAccount", "shadowAccount", "brUser"].forEach(cst => {
                let val3 = {};
                vals3[config_1.ldapConfig.user['class']] = cst;
                basics_1.Basics.change("user", uid, "add", vals3).then(res => {
                    if (!res) {
                        throw "Erreur lors de l'ajout d'une valeur constante à la feuille du nouvel utilisateur.";
                    }
                });
            });
            // Ajout dans les groupes à la catégorie voulue
            for (let cat of config_1.categories) {
                for (let gid of data[cat]) {
                    tools_1.Tools.add(uid, gid, cat);
                }
            }
            return true;
        });
    }
    //------------------------------------------------------------------------------------------------------------------------
    // Fonctions de suppression TBT
    //------------------------------------------------------------------------------------------------------------------------
    /**
     * @memberof LDAP
     * @summary Fonction qui supprime un utilisateur du LDAP.
     * @desc Cette fonction commence par gérer les groupes du membre puis le supprime entièrement.
     * Appelle {@link LDAP.clear} bien sûr, mais aussi {@link Tools.remove} pour gérer les groupes de l'utilisateur sortant.
     * @arg {string} uid - uid de la victime
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    static delete(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                // Gestion des groupes d'abord
                let profil = yield User.peek(uid);
                for (let cat of config_1.categories) {
                    profil[config_1.ldapConfig.user[cat]].forEach(function (gid) {
                        return __awaiter(this, void 0, void 0, function* () {
                            // Enlever de la liste des membres
                            let lm = yield tools_1.Tools.get(gid, "group", cat);
                            if (lm.includes(uid)) {
                                // Supprime tous les membres
                                if (!(yield basics_1.Basics.change("group", gid, "del", config_1.ldapConfig.group[cat]))) {
                                    throw "Erreur lors de la suppression de tous les membres du groupe.";
                                }
                                // Les rajoute un par un, sauf pour le supprimé
                                lm.forEach(id => {
                                    if (id != uid) {
                                        tools_1.Tools.add(id, gid, cat).then(res => {
                                            if (!res) {
                                                throw "Erreur lors du ré-ajout d'un autre membre";
                                            }
                                        });
                                    }
                                });
                            }
                        });
                    });
                }
            }
            catch (err) {
                throw "Erreur lors de l'obtention des informations de l'utilisateur à supprimer.";
            }
            // Elimination
            if (!basics_1.Basics.clear("user", uid)) {
                throw "Erreur lors de la suppression de l'utilisateur.";
            }
            return true;
        });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui édite un utilisateur existant dans le LDAP.
     * @desc Appelle simplement {@link Tools.edit}. Sans effet sur les groupes de l'utilisateur concerné.
     * @arg {userData} data - Dictionnaire des informations utilisateurs
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @async
     * @static
     */
    static edit(data) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return tools_1.Tools.edit("user", data);
            }
            catch (err) {
                throw "Erreur lors de la modification d'un utilisateur.";
            }
        });
    }
}
exports.User = User;
//# sourceMappingURL=user.js.map