"use strict";
/**
 * @file Ce fichier regroupe les fonctions fondamentales aux interactions avec le LDAP.
 * C'est ici que tout le filtrage est opéré, au plus bas niveau.
 * Toutes les fonctions écrites ici sont asynchrones et renvoient des Promises ce qui nécessite de les appeler avec la synthaxe
 * un peu particulière `f(args).then(res => ...)` pour exploiter leur résultat.
 * @author hawkspar
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ldapjs_1 = __importDefault(require("ldapjs"));
// Toutes les entrées utilisateur sont escapées par sécurité
const ldap_escape_1 = __importDefault(require("ldap-escape"));
// Fichier de ldapConfig du ldap
const config_1 = require("./config");
// Connection au serveur LDAP avec des temps de timeout arbitraires
var client = ldapjs_1.default.createClient({ url: config_1.ldapConfig.server });
//------------------------------------------------------------------------------------------------------------------------
// Fonctions de base agissant sur le LDAP
//------------------------------------------------------------------------------------------------------------------------
class Basics {
    /**
     * @class Basics
     * @classdesc Cette classe est la brique de base du fichier tout entier puisqu'elle contient les functions qui agissent directement sur le LDAP.
     * @memberof LDAP
     * @summary Constructeur vide.
     */
    constructor() { }
    /**
     * @memberof LDAP
     * @summary Fonction qui sert à s'identifier sur le LDAP.
     * @desc Assez important en terme de sécurité, de gestion de conflit, et de droit d'accès. Méthode ldapjs
     * (voir [`Client API`](http://ldapjs.org/client.html) méthode bind).
     * @arg {string} dn - Nom de domaine ; identifiant de l'utilisateur cherchant à se connecter
     * @arg {string} password - Mot de passe de l'utilisateur cherchant à se connecter
     * @returns {Promise(boolean)} `true` si l'opération s'est bien déroulée, `false` sinon.
     * @static
     * @async
     */
    static bind(dn, password) {
        return __awaiter(this, void 0, void 0, function* () {
            // Escape DN as everywhere in this file, but password is taken as is
            client.bind(dn, password, res => {
                // Gestion erreur
                try {
                    res;
                }
                catch (err) {
                    throw "Erreur lors de la connection au LDAP.";
                }
            });
            // End with a boolean
            return true;
        });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui sert à s'identifier sur le LDAP avec plein pouvoirs.
     * @desc Appelle {@link bind} avec un utilisateur tout puissant.
     * @returns {Promise(boolean)} `true` si l'opération s'est bien déroulée, `false` sinon.
     * @static
     * @async
     */
    static adminBind() {
        return __awaiter(this, void 0, void 0, function* () { return Basics.bind(config_1.credentialsLdapConfig.dn, config_1.credentialsLdapConfig.password); });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui sert à se déconnecter du LDAP.
     * @desc Assez important en terme de sécurité, de gestion de conflit, et de droit d'accès.
     * Fait appel à {@link Basics.bind} avec deux champs vides.
     * @returns {Promise(boolean)} `true` si l'opération s'est bien déroulée, `false` sinon.
     * @static
     * @async
     */
    static unbind() {
        return __awaiter(this, void 0, void 0, function* () { return Basics.bind("", ""); });
    }
    /**
     * @callback entryHandler
     * @arg entry {*} - Convoluted ldap.js search result object
     */
    /**
     * @memberof LDAP
     * @summary Fonction qui interroge le LDAP selon un protocole spécifié en argument et modifie une liste pour y insérer les valeurs trouvées.
     * @desc Cette fonction utilise ldapjs (voir [`Client API`](http://ldapjs.org/client.html) méthode search). Cette fonction fait une demande au LDAP
     * qu'elle filtre selon un schéma prédéfini dans `filter` et à chaque résultat (event SearchEntry) le met dans une liste, et renvoit la liste à l'issue (event end).
     * @arg {'gr'|'us'} domain - Emplacement de la requête (groupe ou utilisateur)
     * @arg {string[]} attributes - Attributs à renvoyer
     * @arg {string} id [null] - Identifiant facultatif pour une recherche triviale en o(1)
     * @arg {string} filter ["(objectClass=*)"] - Filtre logique de la recherche (format [`RFC2254`](https://tools.ietf.org/search/rfc2254)) déjà passé au ldapEscape
     * @arg {entryHandler} handler - Wrapper pour gérer les requêtes simples ou multiples
     * @return {void} Utilise handler pour gérer ses résultats au fur et à mesure
     * @static
     * @async
     */
    static search(domain, attributes, id, filter, handler) {
        Basics.adminBind();
        let dn = "";
        if (id != null) {
            dn += config_1.ldapConfig.key_id + '=' + ldap_escape_1.default.dn("${txt}", { txt: id }) + ',';
        }
        dn += config_1.ldapConfig.dn[domain];
        // Interrogation LDAP selon filter
        let promise = new Promise(function (resolve, reject) {
            client.search(dn, {
                "scope": "sub",
                "filter": filter,
                "attributes": attributes
            }, (err, res) => {
                // Gestion erreur ; pb car pas simple true / autre en sortie
                if (err) {
                    throw "Erreur lors de la recherche sur le LDAP.";
                }
                else {
                    // Dès que la recherche renvoit une entrée, on stocke les attributs qui nous intéresse
                    res.on('searchEntry', entry => { handler(entry); });
                    // Si la recherche renvoie une erreur, on renvoit
                    res.on('error', resErr => { throw resErr; });
                    // Quand la recherche est finie on se déconnecte
                    res.on('end', _ => { Basics.unbind(); resolve(); });
                }
            });
        });
        return promise;
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui interroge le LDAP selon un protocole spécifié en argument et renvoit une liste de valeurs trouvées.
     * @desc Cette fonction utilise {@link LDAP.search} directement.
     * @arg {'gr'|'us'} domain - Emplacement de la requête (groupe ou utilisateur)
     * @arg {string} attribute - Attribut unique à renvoyer
     * @arg {string} id [null] - Identifiant facultatif pour une recherche triviale en o(1)
     * @arg {string} filter ["(objectClass=*)"] - Filtre logique de la recherche (format [`RFC2254`](https://tools.ietf.org/search/rfc2254)) déjà passé au ldapEscape
     * @return {Promise(string[])} Résultats de la recherche ; soit une liste de valeurs d'attributs,
     * soit une liste de dictionnaires si on veut plus d'un attribut (les clés du dictionnaire sont celles du LDAP)
     * @static
     * @async
     */
    static searchSingle(domain, attribute, id = null, filter = "(objectClass=*)") {
        return __awaiter(this, void 0, void 0, function* () {
            let vals = [];
            yield Basics.search(domain, [attribute], id, filter, entry => {
                // Cas un seul attribut où le résultat est une liste directement
                vals.push(entry.object[attribute]);
            });
            return vals;
        });
    }
    /**
     * @memberof LDAP
     * @summary Fonction qui interroge le LDAP selon un protocole spécifié en argument et renvoit les valeurs trouvées.
     * @desc Cette fonction utilise ldapjs (voir [`Client API`](http://ldapjs.org/client.html) méthode search). Cette fonction fait une demande au LDAP
     * qu'elle filtre selon un schéma prédéfini dans `filter` et à chaque résultat (event SearchEntry) le met dans une liste, et renvoit la liste à l'issue (event end).
     * @arg {'gr'|'us'} domain - Emplacement de la requête (groupe ou utilisateur)
     * @arg {string[]} attributes - Liste des attributs qui figureront dans le résultat final ; peut aussi être un seul élément
     * @arg {string} id [null] - Identifiant facultatif pour une recherche triviale en o(1)
     * @arg {string} filter ["(objectClass=*)"] - Filtre logique de la recherche (format [`RFC2254`](https://tools.ietf.org/search/rfc2254)) déjà passé au ldapEscape
     * @return {Promise(Array<dic>)} Résultats de la recherche ; soit une liste de valeurs d'attributs,
     * soit une liste de dictionnaires si on veut plus d'un attribut (les clés du dictionnaire sont celles du LDAP)
     * @static
     * @async
     */
    static searchMultiple(domain, attributes, id = null, filter = "(objectClass=*)") {
        return __awaiter(this, void 0, void 0, function* () {
            let vals = [];
            yield Basics.search(domain, attributes, id, filter, entry => {
                // Cas plusieurs attributs donc résultat dictionnaire
                vals.push({});
                attributes.forEach(attribute => {
                    vals.slice(-1)[0][attribute] = entry.object[attribute];
                });
            });
            return vals;
        });
    }
    //TBT
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de modifier un élément sur le LDAP. Gestion intelligente de l'appartenance à un binet.
     * @desc Cette fonction traite la demande avec ldapjs (voir [`Client API`](http://ldapjs.org/client.html) méthode modify).
     * @arg {'gr'|'us'} domain - Emplacement de la requête (groupe ou utilisateur)
     * @arg {string} id - Identifiant unique de la feuille à modifier
     * @arg {"add"|"del"|"replace"} op - Operation à réaliser sur le LDAP. Trois opération sont possibles ; "add", qui rajoute des attributs et qui peut créer des doublons,
     * "del" qui en supprime, et "replace" qui remplace du contenu par un autre.
     * @arg {dic} mod - Dictionnaire contenant les attributs à modifier et les nouvelles valeurs des attributs.
     * @arg {string} mod[key] - Nouvelle valeur de l'attribut key. Une nouvelle valeur vide ("") est équivalent à la suppression de cet attribut.
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, `false` sinon.
     * @static
     * @async
     */
    static change(domain, id, op, mod) {
        return __awaiter(this, void 0, void 0, function* () {
            Basics.adminBind();
            let dn = config_1.ldapConfig.key_id + '=' + id + ',';
            dn += config_1.ldapConfig.dn[domain];
            // Modification LDAP selon dn fourni en argument (pourrait prendre une liste de Changes)
            client.modify(ldap_escape_1.default.dn("${txt}", { txt: dn }), new ldapjs_1.default.Change({
                operation: op,
                modification: mod,
            }), err => {
                throw "Erreur lors d'une opération de modification sur le LDAP.";
            });
            Basics.unbind();
            return true;
        });
    }
    //TBT
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de rajouter un élément sur le LDAP.
     * @desc  Cette fonction traite la demande avec ldapjs (voir [`Client API`](http://ldapjs.org/client.html) méthode add).
     * @arg {'gr'|'us'} domain - Emplacement de la requête (groupe ou utilisateur)
     * @arg {Object.<string, string>} vals - Dictionnaire contenant les valeurs à créer (contient un champ en ldapConfig.key_id)
     * @arg {Object} vals[key] - Nouvelle valeur pour le champ key
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon.
     * @static
     * @async
     */
    static add(domain, vals) {
        return __awaiter(this, void 0, void 0, function* () {
            Basics.adminBind();
            let dn = config_1.ldapConfig.key_id + "=" + vals[config_1.ldapConfig.key_id];
            dn += config_1.ldapConfig.dn[domain];
            // Ajout LDAP selon la ldapConfiguration en argument
            client.add(ldap_escape_1.default.dn("${txt}", { txt: dn }), vals, err => {
                throw "Erreur lors d'une opération d'ajout sur le LDAP.";
            });
            Basics.unbind();
            return true;
        });
    }
    //TBT
    /**
     * @memberof LDAP
     * @summary Fonction qui permet de supprimer une feuille du LDAP.
     * @desc Cette fonction traite la demande avec ldapjs (voir [`Client API`](http://ldapjs.org/client.html) méthode del).
     * Elle est différente de modify avec "del" car elle affecte directement une feuille et pas un attribut.
     * @arg {'gr'|'us'} domain - Emplacement de la requête (groupe ou utilisateur)
     * @arg {string} id - Identifiant unique de la cible
     * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
     * @static
     * @async
     */
    static clear(domain, id) {
        return __awaiter(this, void 0, void 0, function* () {
            Basics.adminBind();
            let dn = config_1.ldapConfig.key_id + '=' + id + ',';
            dn += config_1.ldapConfig.dn[domain];
            // Suppression LDAP
            client.del(ldap_escape_1.default.dn("${txt}", { txt: dn }), err => {
                throw "Erreur lors d'une opération de suppression sur le LDAP.";
            });
            Basics.unbind();
            return true;
        });
    }
}
exports.Basics = Basics;
//# sourceMappingURL=basics.js.map